@@ -1,6 +1,6 @@
 {-|
 
-This module can automatically generate TopEntity definitions from 'Clash.NamedTypes'
+This module can automatically generate TopEntity definitions from "Clash.NamedTypes"
 annotations. Annotations involving data\/type families must be inspected for correctness.
 Not all cases can be handled with automatic generation due to the difficulty of type manipulation
 in template Haskell. In particular annotations __inside__ the following is unlikely to work:
@@ -90,6 +90,7 @@ import           Data.Generics.Uniplate.Data    (rewrite)
 import           Clash.Annotations.TopEntity    ( PortName(..)
                                                 , TopEntity(..)
                                                 )
+import           Clash.Magic                    (SimOnly)
 import           Clash.NamedTypes               ((:::))
 import           Clash.Signal                   ( HiddenClockResetEnable
                                                 , HiddenClock, HiddenReset, HiddenEnable
@@ -225,6 +226,7 @@ datatypeNameToPorts name = do
   constructors <- tryReifyDatatype [] datatypeCons name
 
   names <- case constructors of
+    _ | name == ''SimOnly -> return $ Complete []
     []  -> return $ Complete []
     [x] -> portsFromTypes (constructorFields x)
     xs  -> handleNamesInSum xs
@@ -283,6 +285,8 @@ typeTreeToPorts (ConTF name) = do
 typeTreeToPorts f@(AppTF (a,a') (b,b')) = do
   -- Gather types applied to a head type
   case unapp (AppT a b) of
+    -- Skip SimOnly constructs - they're always zero bits
+    (ConT x : _) | x == ''SimOnly -> pure (Complete [])
     -- Return the inner type for signals
     (ConT x : _ : _ : []) | x == ''Clash.Signal.Signal -> b'
     (ConT x : _ : _ : _ : []) | x == ''Clash.Signal.Delayed.DSignal -> b'
@@ -372,7 +376,7 @@ typeTreeToPorts f@(AppTF (a,a') (b,b')) = do
   applyFamilyBindings _ _ _ = error "familyTyMap called with non family argument!"
 
 #if MIN_VERSION_template_haskell(2,15,0)
-  tySynArgs (TySynEqn _ args _) = tail (unapp args)
+  tySynArgs (TySynEqn _ args _) = drop 1 (unapp args)
 #else
   tySynArgs (TySynEqn args _) = args
 #endif
