@@ -1,7 +1,8 @@
 {-|
 Copyright  :  (C) 2019, Myrtle Software Ltd
+                  2024, QBayLogic B.V.
 License    :  BSD2 (see the file LICENSE)
-Maintainer :  Christiaan Baaij <christiaan.baaij@gmail.com>
+Maintainer :  QBayLogic B.V. <devops@qaylogic.com>
 -}
 
 {-# LANGUAGE CPP #-}
@@ -10,7 +11,16 @@ Maintainer :  Christiaan Baaij <christiaan.baaij@gmail.com>
 
 {-# OPTIONS_GHC -fplugin GHC.TypeLits.KnownNat.Solver #-}
 
-module Clash.Class.Exp (Exp, ExpResult, (^)) where
+module Clash.Class.Exp
+  ( Exp
+  , ExpResult
+  , (^)
+
+  -- | * Internal
+  , expIndex#
+  , expSigned#
+  , expUnsigned#
+  ) where
 
 import qualified Prelude                       as P
 import           Prelude                       hiding ((^))
@@ -38,6 +48,8 @@ class Exp a where
     -> ExpResult a n
     -- ^ Resized result, guaranteed to not have overflown
 
+infixr 8 ^
+
 instance KnownNat m => Exp (Index m) where
   type ExpResult (Index m) n = Index (Max 2 (m ^ n))
 
