@@ -2,7 +2,9 @@
 Copyright  :  (C) 2013-2016, University of Twente,
                   2016-2017, Myrtle Software Ltd,
                   2017     , Google Inc.,
-                  2021-2022, QBayLogic B.V.
+                  2021-2023, QBayLogic B.V.,
+                  2022     , Google Inc.,
+                  2024     , Alex Mason,
 License    :  BSD2 (see the file LICENSE)
 Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 
@@ -36,7 +38,7 @@ data Instruction
   | Load MemAddr Reg
   | Store Reg MemAddr
   | Nop
-  deriving (Eq, Show)
+  deriving (Eq, Show, Generic, NFDataX)
 
 data Reg
   = Zero
@@ -49,7 +51,7 @@ data Reg
   deriving (Eq, Show, Enum, Generic, NFDataX)
 
 data Operator = Add | Sub | Incr | Imm | CmpGt
-  deriving (Eq, Show)
+  deriving (Eq, Show, Generic, NFDataX)
 
 data MachCode
   = MachCode
@@ -86,7 +88,7 @@ cpu
      , (MemAddr, Maybe (MemAddr,Value), InstrAddr)
      )
 cpu regbank (memOut, instr) =
-  (regbank', (rdAddr, (,aluOut) '<$>' wrAddrM, fromIntegral ipntr))
+  (regbank', (rdAddr, (,aluOut) '<$>' wrAddrM, bitCoerce ipntr))
  where
   -- Current instruction pointer
   ipntr = regbank 'Clash.Sized.Vector.!!' PC
@@ -279,7 +281,7 @@ cpu2
      , (MemAddr, Maybe (MemAddr,Value), InstrAddr)
      )
 cpu2 (regbank, ldRegD) (memOut, instr) =
-  ((regbank', ldRegD'), (rdAddr, (,aluOut) '<$>' wrAddrM, fromIntegral ipntr))
+  ((regbank', ldRegD'), (rdAddr, (,aluOut) '<$>' wrAddrM, bitCoerce ipntr))
  where
   -- Current instruction pointer
   ipntr = regbank 'Clash.Sized.Vector.!!' PC
@@ -385,9 +387,12 @@ This concludes the short introduction to using 'blockRam'.
 
 -}
 
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveAnyClass #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE QuasiQuotes #-}
+{-# LANGUAGE TemplateHaskellQuotes #-}
 
 {-# LANGUAGE Trustworthy #-}
 
@@ -422,41 +427,47 @@ module Clash.Explicit.BlockRam
   )
 where
 
-import           Clash.HaskellPrelude
+import Clash.HaskellPrelude
 
-import           Control.Exception      (catch, throw)
-import           Control.Monad          (forM_)
-import           Control.Monad.ST       (ST, runST)
-import           Control.Monad.ST.Unsafe (unsafeInterleaveST, unsafeIOToST, unsafeSTToIO)
-import           Data.Array.MArray      (newListArray)
-import qualified Data.List              as L
-import           Data.Maybe             (isJust, fromMaybe)
-import           GHC.Arr
-  (STArray, unsafeReadSTArray, unsafeWriteSTArray)
-import qualified Data.Sequence          as Seq
-import           Data.Sequence          (Seq)
-import           Data.Tuple             (swap)
-import           GHC.Generics           (Generic)
-import           GHC.Stack              (HasCallStack, withFrozenCallStack)
-import           GHC.TypeLits           (KnownNat, type (^), type (<=))
-import           Unsafe.Coerce          (unsafeCoerce)
+import Control.Exception (catch, throw)
+import Control.Monad (forM_)
+import Control.Monad.ST (ST, runST)
+import Control.Monad.ST.Unsafe (unsafeInterleaveST, unsafeIOToST, unsafeSTToIO)
+import Data.Array.MArray (newListArray)
+import Data.List.Infinite (Infinite(..), (...))
+import Data.Maybe (isJust)
+import Data.Sequence (Seq)
+import Data.String.Interpolate (__i)
+import GHC.Arr (STArray, unsafeReadSTArray, unsafeWriteSTArray)
+import GHC.Generics (Generic)
+import GHC.Stack (HasCallStack, withFrozenCallStack)
+import GHC.TypeLits (KnownNat, type (^), type (<=))
+import Unsafe.Coerce (unsafeCoerce)
 
-import           Clash.Annotations.Primitive
-  (hasBlackBox)
-import           Clash.Class.Num        (SaturationMode(SatBound), satSucc)
-import           Clash.Explicit.Signal  (KnownDomain, Enable, register, fromEnable)
-import           Clash.Signal.Internal
+import Clash.Annotations.Primitive
+  (Primitive(InlineYamlPrimitive), HDL(..), hasBlackBox)
+import Clash.Class.AutoReg (AutoReg(autoReg))
+import Clash.Class.BitPack (bitToBool, msb)
+import Clash.Class.Num (SaturationMode(SatBound), satSucc)
+import Clash.Explicit.BlockRam.Model (TdpbramModelConfig(..), tdpbramModel)
+import Clash.Explicit.Signal (KnownDomain, Enable, register, fromEnable, andEnable)
+import Clash.Promoted.Nat (SNat(..))
+import Clash.Signal.Bundle (unbundle)
+import Clash.Signal.Internal
   (Clock(..), Reset, Signal (..), invertReset, (.&&.), mux)
-import           Clash.Promoted.Nat     (SNat(..), snatToNum, natToNum)
-import           Clash.Signal.Bundle    (unbundle, bundle)
-import           Clash.Signal.Internal.Ambiguous (clockPeriod)
-import           Clash.Sized.Unsigned   (Unsigned)
-import           Clash.Sized.Index      (Index)
-import           Clash.Sized.Vector     (Vec, replicate, iterateI)
-import qualified Clash.Sized.Vector     as CV
-import           Clash.XException
-  (maybeIsX, NFDataX(deepErrorX), defaultSeqX, fromJustX, undefined,
-   XException (..), seqX, isX, errorX)
+import Clash.Sized.BitVector (BitVector)
+import Clash.Sized.Index (Index)
+import Clash.Sized.Unsigned (Unsigned)
+import Clash.Sized.Vector (Vec, replicate, iterateI)
+import Clash.XException
+ (maybeIsX, NFDataX(deepErrorX), defaultSeqX, fromJustX, undefined,
+ XException (..), seqX, errorX)
+import Clash.XException.MaybeX (MaybeX(..), andX)
+
+import qualified Data.Sequence as Seq
+import qualified Data.List as L
+
+import qualified Clash.Sized.Vector as CV
 
 {- $tdpbram
 A true dual-port block RAM has two fully independent, fully functional access
@@ -494,7 +505,7 @@ data Reg
 
 >>> :{
 data Operator = Add | Sub | Incr | Imm | CmpGt
-  deriving (Eq,Show)
+  deriving (Eq, Show, Generic, NFDataX)
 :}
 
 >>> :{
@@ -505,7 +516,7 @@ data Instruction
   | Load MemAddr Reg
   | Store Reg MemAddr
   | Nop
-  deriving (Eq,Show)
+  deriving (Eq, Show, Generic, NFDataX)
 :}
 
 >>> :{
@@ -543,7 +554,7 @@ let cpu :: Vec 7 Value          -- ^ Register bank
         -> ( Vec 7 Value
            , (MemAddr,Maybe (MemAddr,Value),InstrAddr)
            )
-    cpu regbank (memOut,instr) = (regbank',(rdAddr,(,aluOut) <$> wrAddrM,fromIntegral ipntr))
+    cpu regbank (memOut,instr) = (regbank',(rdAddr,(,aluOut) <$> wrAddrM,bitCoerce ipntr))
       where
         -- Current instruction pointer
         ipntr = regbank C.!! PC
@@ -660,7 +671,7 @@ let cpu2 :: (Vec 7 Value,Reg)    -- ^ (Register bank, Load reg addr)
          -> ( (Vec 7 Value,Reg)
             , (MemAddr,Maybe (MemAddr,Value),InstrAddr)
             )
-    cpu2 (regbank,ldRegD) (memOut,instr) = ((regbank',ldRegD'),(rdAddr,(,aluOut) <$> wrAddrM,fromIntegral ipntr))
+    cpu2 (regbank,ldRegD) (memOut,instr) = ((regbank',ldRegD'),(rdAddr,(,aluOut) <$> wrAddrM,bitCoerce ipntr))
       where
         -- Current instruction pointer
         ipntr = regbank C.!! PC
@@ -770,7 +781,8 @@ blockRam
   :: ( KnownDomain dom
      , HasCallStack
      , NFDataX a
-     , Enum addr )
+     , Enum addr
+     , NFDataX addr )
   => Clock dom
   -- ^ 'Clock' to synchronize to
   -> Enable dom
@@ -854,6 +866,7 @@ blockRamU
      , HasCallStack
      , NFDataX a
      , Enum addr
+     , NFDataX addr
      , 1 <= n )
   => Clock dom
   -- ^ 'Clock' to synchronize to
@@ -934,7 +947,8 @@ blockRamU# clk en SNat =
     (CV.map
       (\i -> deepErrorX $ "Initial value at index " <> show i <> " undefined.")
       (iterateI @n succ (0 :: Int)))
-{-# NOINLINE blockRamU# #-}
+-- See: https://github.com/clash-lang/clash-compiler/pull/2511
+{-# CLASH_OPAQUE blockRamU# #-}
 {-# ANN blockRamU# hasBlackBox #-}
 
 -- | A version of 'blockRam' that is initialized with the same value on all
@@ -945,6 +959,7 @@ blockRam1
      , HasCallStack
      , NFDataX a
      , Enum addr
+     , NFDataX addr
      , 1 <= n )
   => Clock dom
   -- ^ 'Clock' to synchronize to
@@ -1022,7 +1037,8 @@ blockRam1#
 blockRam1# clk en n a =
   -- TODO: Generalize to single BRAM primitive taking an initialization function
   blockRam# clk en (replicate n a)
-{-# NOINLINE blockRam1# #-}
+-- See: https://github.com/clash-lang/clash-compiler/pull/2511
+{-# CLASH_OPAQUE blockRam1# #-}
 {-# ANN blockRam1# hasBlackBox #-}
 
 -- | blockRAM primitive
@@ -1049,7 +1065,7 @@ blockRam#
   -- ^ Value to write (at address @w@)
   -> Signal dom a
   -- ^ Value of the BRAM at address @r@ from the previous clock cycle
-blockRam# (Clock _) gen content = \rd wen waS wd -> runST $ do
+blockRam# (Clock _ Nothing) gen content = \rd wen waS wd -> runST $ do
   ramStart <- newListArray (0,szI-1) contentL
   -- start benchmark only
   -- ramStart <- unsafeThawSTArray ramArr
@@ -1117,8 +1133,10 @@ blockRam# (Clock _) gen content = \rd wen waS wd -> runST $ do
                              " not in range [0.." <> show szI <> ")"))
        in forM_ [0..(szI-1)] (\j -> unsafeWriteSTArray s j d)
   {-# INLINE safeUpdate #-}
+blockRam# _ _ _ = error "blockRam#: dynamic clocks not supported"
 {-# ANN blockRam# hasBlackBox #-}
-{-# NOINLINE blockRam# #-}
+-- See: https://github.com/clash-lang/clash-compiler/pull/2511
+{-# CLASH_OPAQUE blockRam# #-}
 
 -- | Create a read-after-write block RAM from a read-before-write one
 readNew
@@ -1154,6 +1172,47 @@ data RamOp n a
   -- ^ No operation
   deriving (Generic, NFDataX, Show)
 
+instance (AutoReg a, KnownNat n) => AutoReg (RamOp n a) where
+  autoReg clk rst en initVal input =
+    createRamOp <$> tagR <*> valAddr <*> valValue
+    where
+      tag = toTag <$> input
+
+      toTag op = case op of
+        RamNoOp    -> 0b00 :: BitVector 2
+        RamRead{}  -> 0b01
+        RamWrite{} -> 0b10
+
+      tagInit = toTag initVal
+      tagR = register clk rst en tagInit tag
+
+      toAddr op = case op of
+        RamNoOp         -> deepErrorX "autoReg'.ramOpAddr"
+        RamRead addr    -> addr
+        RamWrite addr _ -> addr
+
+      toValue op = case op of
+        RamWrite _ a -> a
+        _ -> deepErrorX "autoReg'.ramOpValue"
+
+
+      opAddr  = toAddr  <$> input
+      opValue = toValue <$> input
+
+      addrInit = toAddr  initVal
+      valInit  = toValue initVal
+
+      valAddr  = autoReg clk rst (andEnable en           ((/=0) <$> tag)) addrInit opAddr
+      valValue = autoReg clk rst (andEnable en (bitToBool . msb <$> tag)) valInit opValue
+
+      createRamOp t addr val = case t of
+        0b00 -> RamNoOp
+        0b01 -> RamRead addr
+        0b10 -> RamWrite addr val
+        _ -> deepErrorX  "autoReg'.createRamOp: impossible"
+
+  {-# INLINE autoReg #-}
+
 ramOpAddr :: RamOp n a -> Index n
 ramOpAddr (RamRead addr)    = addr
 ramOpAddr (RamWrite addr _) = addr
@@ -1204,20 +1263,6 @@ trueDualPortBlockRam = \clkA clkB opA opB ->
     clkA (isOp <$> opA) (isRamWrite <$> opA) (ramOpAddr <$> opA) (fromJustX . ramOpWriteVal <$> opA)
     clkB (isOp <$> opB) (isRamWrite <$> opB) (ramOpAddr <$> opB) (fromJustX . ramOpWriteVal <$> opB)
 
-toMaybeX :: a -> MaybeX a
-toMaybeX a =
-  case isX a of
-    Left _ -> IsX
-    Right _ -> IsDefined a
-
-data MaybeX a = IsX | IsDefined !a
-
-data Conflict = Conflict
-  { cfRWA     :: !(MaybeX Bool) -- ^ Read/Write conflict for output A
-  , cfRWB     :: !(MaybeX Bool) -- ^ Read/Write conflict for output B
-  , cfWW      :: !(MaybeX Bool) -- ^ Write/Write conflict
-  , cfAddress :: !(MaybeX Int) }
-
 -- [Note: eta port names for trueDualPortBlockRam]
 --
 -- By naming all the arguments and setting the -fno-do-lambda-eta-expansion GHC
@@ -1233,9 +1278,214 @@ data Conflict = Conflict
 -- into its own module / architecture.
 trueDualPortBlockRamWrapper clkA enA weA addrA datA clkB enB weB addrB datB =
   trueDualPortBlockRam# clkA enA weA addrA datA clkB enB weB addrB datB
-{-# NOINLINE trueDualPortBlockRamWrapper #-}
+-- See: https://github.com/clash-lang/clash-compiler/pull/2511
+{-# CLASH_OPAQUE trueDualPortBlockRamWrapper #-}
 
--- | Primitive of 'trueDualPortBlockRam'.
+-- See: https://github.com/clash-lang/clash-compiler/pull/2511
+{-# CLASH_OPAQUE trueDualPortBlockRam# #-}
+{-# ANN trueDualPortBlockRam# hasBlackBox #-}
+{-# ANN trueDualPortBlockRam# (
+  let
+    bbName = show 'trueDualPortBlockRam#
+    _hasCallStack
+     :< knownNatAddrs
+     :< _knownDomainA
+     :< _knownDomainB
+     :< _nfdataX
+
+     :< clockA
+     :< enaA
+     :< wenaA
+     :< addrA
+     :< datA
+
+     :< clockB
+     :< enaB
+     :< wenaB
+     :< addrB
+     :< datB
+
+     :< _ = ((0 :: Int)...)
+
+    symBlockName
+     :< symDoutA
+     :< symDoutB
+     :< _ = ((0 :: Int)...)
+  in InlineYamlPrimitive [VHDL] [__i|
+    BlackBox:
+      name: "#{bbName}"
+      kind: Declaration
+      template: |-
+        -- trueDualPortBlockRam begin
+        ~GENSYM[~RESULT_trueDualPortBlockRam][#{symBlockName}] : block
+          -- Shared memory
+          type mem_type is array ( ~LIT[#{knownNatAddrs}]-1 downto 0 ) of ~TYP[#{datA}];
+          shared variable mem : mem_type;
+          signal ~GENSYM[a_dout][#{symDoutA}] : ~TYP[#{datA}];
+          signal ~GENSYM[b_dout][#{symDoutB}] : ~TYP[#{datB}];
+        begin
+
+          -- Port A
+          process(~ARG[#{clockA}])
+          begin
+              if(rising_edge(~ARG[#{clockA}])) then
+                    if(~ARG[#{enaA}]) then
+                      if(~ARG[#{wenaA}]) then
+                          mem(~IF~SIZE[~TYP[#{addrA}]]~THENto_integer(~ARG[#{addrA}])~ELSE0~FI) := ~ARG[#{datA}];
+                      end if;
+                      ~SYM[#{symDoutA}] <= mem(~IF~SIZE[~TYP[#{addrA}]]~THENto_integer(~ARG[#{addrA}])~ELSE0~FI);
+                  end if;
+              end if;
+          end process;
+
+          -- Port B
+          process(~ARG[#{clockB}])
+          begin
+              if(rising_edge(~ARG[#{clockB}])) then
+                  if(~ARG[#{enaB}]) then
+                      if(~ARG[#{wenaB}]) then
+                          mem(~IF~SIZE[~TYP[#{addrB}]]~THENto_integer(~ARG[#{addrB}])~ELSE0~FI) := ~ARG[#{datB}];
+                      end if;
+                      ~SYM[#{symDoutB}] <= mem(~IF~SIZE[~TYP[#{addrB}]]~THENto_integer(~ARG[#{addrB}])~ELSE0~FI);
+                  end if;
+              end if;
+          end process;
+
+          ~RESULT <= (~SYM[#{symDoutA}], ~SYM[#{symDoutB}]);
+        end block;
+        -- end trueDualPortBlockRam
+|]) #-}
+{-# ANN trueDualPortBlockRam# (
+  let
+    bbName = show 'trueDualPortBlockRam#
+    _hasCallStack
+     :< knownNatAddrs
+     :< knownDomainA
+     :< knownDomainB
+     :< _nfdataX
+
+     :< clockA
+     :< enaA
+     :< wenaA
+     :< addrA
+     :< datA
+
+     :< clockB
+     :< enaB
+     :< wenaB
+     :< addrB
+     :< datB
+
+     :< _ = ((0 :: Int)...)
+    symMem
+     :< symDoutA
+     :< symDoutB
+     :< _ = ((0 :: Int)...)
+  in InlineYamlPrimitive [SystemVerilog] [__i|
+    BlackBox:
+      name: "#{bbName}"
+      kind: Declaration
+      template: |-
+        // trueDualPortBlockRam begin
+        // Shared memory
+        logic [~SIZE[~TYP[#{datA}]]-1:0] ~GENSYM[mem][#{symMem}] [~LIT[#{knownNatAddrs}]-1:0];
+
+        ~SIGD[~GENSYM[a_dout][#{symDoutA}]][#{datA}];
+        ~SIGD[~GENSYM[b_dout][#{symDoutB}]][#{datB}];
+
+        // Port A
+        always @(~IF~ACTIVEEDGE[Rising][#{knownDomainA}]~THENposedge~ELSEnegedge~FI ~ARG[#{clockA}]) begin
+            if(~ARG[#{enaA}]) begin
+                ~SYM[#{symDoutA}] <= ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrA}]]~THEN~ARG[#{addrA}]~ELSE0~FI];
+                if(~ARG[#{wenaA}]) begin
+                    ~SYM[#{symDoutA}] <= ~ARG[#{datA}];
+                    ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrA}]]~THEN~ARG[#{addrA}]~ELSE0~FI] <= ~ARG[#{datA}];
+                end
+            end
+        end
+
+        // Port B
+        always @(~IF~ACTIVEEDGE[Rising][#{knownDomainB}]~THENposedge~ELSEnegedge~FI ~ARG[#{clockB}]) begin
+            if(~ARG[#{enaB}]) begin
+                ~SYM[#{symDoutB}] <= ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrB}]]~THEN~ARG[#{addrB}]~ELSE0~FI];
+                if(~ARG[#{wenaB}]) begin
+                    ~SYM[#{symDoutB}] <= ~ARG[#{datB}];
+                    ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrB}]]~THEN~ARG[#{addrB}]~ELSE0~FI] <= ~ARG[#{datB}];
+                end
+            end
+        end
+
+        assign ~RESULT = {~SYM[#{symDoutA}], ~SYM[#{symDoutB}]};
+        // end trueDualPortBlockRam
+|]) #-}
+{-# ANN trueDualPortBlockRam# (
+  let
+    bbName = show 'trueDualPortBlockRam#
+    _hasCallStack
+     :< knownNatAddrs
+     :< knownDomainA
+     :< knownDomainB
+     :< _nfdataX
+
+     :< clockA
+     :< enaA
+     :< wenaA
+     :< addrA
+     :< datA
+
+     :< clockB
+     :< enaB
+     :< wenaB
+     :< addrB
+     :< datB
+
+     :< _ = ((0 :: Int)...)
+
+    symMem
+     :< symDoutA
+     :< symDoutB
+     :< _ = ((0 :: Int)...)
+  in InlineYamlPrimitive [Verilog] [__i|
+    BlackBox:
+      name: "#{bbName}"
+      kind: Declaration
+      template: |-
+        // trueDualPortBlockRam begin
+        // Shared memory
+        reg [~SIZE[~TYP[#{datA}]]-1:0] ~GENSYM[mem][#{symMem}] [~LIT[#{knownNatAddrs}]-1:0];
+
+        reg ~SIGD[~GENSYM[a_dout][#{symDoutA}]][#{datA}];
+        reg ~SIGD[~GENSYM[b_dout][#{symDoutB}]][#{datB}];
+
+        // Port A
+        always @(~IF~ACTIVEEDGE[Rising][#{knownDomainA}]~THENposedge~ELSEnegedge~FI ~ARG[#{clockA}]) begin
+            if(~ARG[#{enaA}]) begin
+                ~SYM[#{symDoutA}] <= ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrA}]]~THEN~ARG[#{addrA}]~ELSE0~FI];
+                if(~ARG[#{wenaA}]) begin
+                    ~SYM[#{symDoutA}] <= ~ARG[#{datA}];
+                    ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrA}]]~THEN~ARG[#{addrA}]~ELSE0~FI] <= ~ARG[#{datA}];
+                end
+            end
+        end
+
+        // Port B
+        always @(~IF~ACTIVEEDGE[Rising][#{knownDomainB}]~THENposedge~ELSEnegedge~FI ~ARG[#{clockB}]) begin
+            if(~ARG[#{enaB}]) begin
+                ~SYM[#{symDoutB}] <= ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrB}]]~THEN~ARG[#{addrB}]~ELSE0~FI];
+                if(~ARG[#{wenaB}]) begin
+                    ~SYM[#{symDoutB}] <= ~ARG[#{datB}];
+                    ~SYM[#{symMem}][~IF~SIZE[~TYP[#{addrB}]]~THEN~ARG[#{addrB}]~ELSE0~FI] <= ~ARG[#{datB}];
+                end
+            end
+        end
+
+        assign ~RESULT = {~SYM[#{symDoutA}], ~SYM[#{symDoutB}]};
+
+        // end trueDualPortBlockRam
+|]) #-}
+
+-- | Primitive for 'trueDualPortBlockRam'
+--
 trueDualPortBlockRam#, trueDualPortBlockRamWrapper ::
   forall nAddrs domA domB a .
   ( HasCallStack
@@ -1243,228 +1493,43 @@ trueDualPortBlockRam#, trueDualPortBlockRamWrapper ::
   , KnownDomain domA
   , KnownDomain domB
   , NFDataX a
-  )
-  => Clock domA
-  -- ^ Clock for port A
-  -> Signal domA Bool
-  -- ^ Enable for port A
-  -> Signal domA Bool
-  -- ^ Write enable for port A
-  -> Signal domA (Index nAddrs)
-  -- ^ Address to read from or write to on port A
-  -> Signal domA a
-  -- ^ Data in for port A; ignored when /write enable/ is @False@
-
-  -> Clock domB
-  -- ^ Clock for port B
-  -> Signal domB Bool
-  -- ^ Enable for port B
-  -> Signal domB Bool
-  -- ^ Write enable for port B
-  -> Signal domB (Index nAddrs)
-  -- ^ Address to read from or write to on port B
-  -> Signal domB a
-  -- ^ Data in for port B; ignored when /write enable/ is @False@
-
-  -> (Signal domA a, Signal domB a)
-  -- ^ Outputs data on /next/ cycle. If write enable is @True@, the data written
-  -- will be echoed. If write enable is @False@, the read data is returned. If
-  -- port enable is @False@, it is /undefined/.
-trueDualPortBlockRam# clkA enA weA addrA datA clkB enB weB addrB datB
-  | snatToNum @Int (clockPeriod @domA) < snatToNum @Int (clockPeriod @domB)
-  = swap (trueDualPortBlockRamModel labelB clkB enB weB addrB datB labelA clkA enA weA addrA datA)
-  | otherwise
-  =       trueDualPortBlockRamModel labelA clkA enA weA addrA datA labelB clkB enB weB addrB datB
- where
-  labelA = "Port A"
-  labelB = "Port B"
-{-# NOINLINE trueDualPortBlockRam# #-}
-{-# ANN trueDualPortBlockRam# hasBlackBox #-}
-
-
--- | Haskell model for the primitive 'trueDualPortBlockRam#'.
---
--- Warning: this model only works if @domFast@'s clock is faster (or equal to)
--- @domSlow@'s clock.
-trueDualPortBlockRamModel ::
-  forall nAddrs domFast domSlow a .
-  ( HasCallStack
-  , KnownNat nAddrs
-  , KnownDomain domSlow
-  , KnownDomain domFast
-  , NFDataX a
   ) =>
 
-  String ->
-  Clock domSlow ->
-  Signal domSlow Bool ->
-  Signal domSlow Bool ->
-  Signal domSlow (Index nAddrs) ->
-  Signal domSlow a ->
+  Clock domA ->
+  -- | Enable
+  Signal domA Bool ->
+  -- | Write enable
+  Signal domA Bool ->
+  -- | Address
+  Signal domA (Index nAddrs) ->
+  -- | Write data
+  Signal domA a ->
 
-  String ->
-  Clock domFast ->
-  Signal domFast Bool ->
-  Signal domFast Bool ->
-  Signal domFast (Index nAddrs) ->
-  Signal domFast a ->
+  Clock domB ->
+  -- | Enable
+  Signal domB Bool ->
+  -- | Write enable
+  Signal domB Bool ->
+  -- | Address
+  Signal domB (Index nAddrs) ->
+  -- | Write data
+  Signal domB a ->
 
-  (Signal domSlow a, Signal domFast a)
-trueDualPortBlockRamModel labelA !_clkA enA weA addrA datA labelB !_clkB enB weB addrB datB =
-  ( startA :- outA
-  , startB :- outB )
+  (Signal domA a, Signal domB a)
+trueDualPortBlockRam# clkA enA weA addrA datA clkB enB weB addrB datB =
+  tdpbramModel
+    TdpbramModelConfig
+      { tdpIsActiveWriteEnable = id
+      , tdpMergeWriteEnable = andX
+      , tdpUpdateRam = updateRam }
+    clkA enA addrA weA datA
+    clkB enB addrB weB datB
  where
-  (outA, outB) =
-    go
-      (Seq.fromFunction (natToNum @nAddrs) initElement)
-      tB -- ensure 'go' hits fast clock first for 1 cycle, then execute slow
-         -- clock for 1 cycle, followed by the regular cadence of 'ceil(tA / tB)'
-         -- cycles for the fast clock followed by 1 cycle of the slow clock
-      (bundle (enA, weA, fromIntegral <$> addrA, datA))
-      (bundle (enB, weB, fromIntegral <$> addrB, datB))
-      startA startB
-
-  tA = snatToNum @Int (clockPeriod @domSlow)
-  tB = snatToNum @Int (clockPeriod @domFast)
-
-  startA = deepErrorX $ "trueDualPortBlockRam: " <> labelA <> ": First value undefined"
-  startB = deepErrorX $ "trueDualPortBlockRam: " <> labelB <> ": First value undefined"
-
-  initElement :: Int -> a
-  initElement n =
-    deepErrorX ("Unknown initial element; position " <> show n)
-
-  unknownEnableAndAddr :: String -> String -> Int -> a
-  unknownEnableAndAddr enaMsg addrMsg n =
-    deepErrorX ("Write enable and data unknown; position " <> show n <>
-                "\nWrite enable error message: " <> enaMsg <>
-                "\nAddress error message: " <> addrMsg)
-
-  unknownAddr :: String -> Int -> a
-  unknownAddr msg n =
-    deepErrorX ("Write enabled, but address unknown; position " <> show n <>
-                "\nAddress error message: " <> msg)
-
-  getConflict :: Bool -> Bool -> Bool -> Int -> Bool -> Int -> Maybe Conflict
-  getConflict enA_ enB_ wenA addrA_ wenB addrB_ =
-    -- If port A or port B is writing on (potentially!) the same address,
-    -- there's a conflict
-    if sameAddr then Just conflict else Nothing
-   where
-    wenAX = toMaybeX wenA
-    wenBX = toMaybeX wenB
-
-    mergeX IsX b = b
-    mergeX a IsX = a
-    mergeX (IsDefined a) (IsDefined b) = IsDefined (a && b)
-
-    conflict = Conflict
-      { cfRWA     = if enB_ then wenBX else IsDefined False
-      , cfRWB     = if enA_ then wenAX else IsDefined False
-      , cfWW      = if enA_ && enB_ then mergeX wenAX wenBX else IsDefined False
-      , cfAddress = toMaybeX addrA_ }
-
-    sameAddr =
-      case (isX addrA_, isX addrB_) of
-        (Left _, _) -> True
-        (_, Left _) -> True
-        _           -> addrA_ == addrB_
-
-  writeRam :: Bool -> Int -> a -> Seq a -> (Maybe a, Seq a)
-  writeRam enable addr dat mem
-    | Left enaMsg <- enableUndefined
-    , Left addrMsg <- addrUndefined
-    = let msg = "Unknown enable and address" <>
-                "\nWrite enable error message: " <> enaMsg <>
-                "\nAddress error message: " <> addrMsg
-       in ( Just (deepErrorX msg)
-          , Seq.fromFunction (natToNum @nAddrs)
-                             (unknownEnableAndAddr enaMsg addrMsg) )
-    | Left enaMsg <- enableUndefined
-    = let msg = "Write enable unknown; position" <> show addr <>
-                "\nWrite enable error message: " <> enaMsg
-       in writeRam True addr (deepErrorX msg) mem
-    | enable
-    , Left addrMsg <- addrUndefined
-    = ( Just (deepErrorX "Unknown address")
-      , Seq.fromFunction (natToNum @nAddrs) (unknownAddr addrMsg) )
-    | enable
-    = (Just dat, Seq.update addr dat mem)
-    | otherwise
-    = (Nothing, mem)
-   where
-    enableUndefined = isX enable
-    addrUndefined = isX addr
-
-  go ::
-    Seq a ->
-    Int ->
-    Signal domSlow (Bool, Bool, Int, a) ->
-    Signal domFast (Bool, Bool, Int, a) ->
-    a -> a ->
-    (Signal domSlow a, Signal domFast a)
-  go ram0 relativeTime as0 bs0 =
-    case compare relativeTime 0 of
-      LT -> goSlow
-      EQ -> goBoth
-      GT -> goFast
-   where
-    (enA_, weA_, addrA_, datA_) :- as1 = as0
-    (enB_, weB_, addrB_, datB_) :- bs1 = bs0
-
-    goBoth prevA prevB = outA2 `seqX` outB2 `seqX` (outA2 :- as2, outB2 :- bs2)
-     where
-      conflict = getConflict enA_ enB_ weA_ addrA_ weB_ addrB_
-
-      (datA1_,datB1_) = case conflict of
-        Just Conflict{cfWW=IsDefined True} ->
-          ( deepErrorX "trueDualPortBlockRam: conflicting write/write queries"
-          , deepErrorX "trueDualPortBlockRam: conflicting write/write queries" )
-        Just Conflict{cfWW=IsX} ->
-          ( deepErrorX "trueDualPortBlockRam: conflicting write/write queries"
-          , deepErrorX "trueDualPortBlockRam: conflicting write/write queries" )
-        _ -> (datA_,datB_)
-
-      (wroteA,ram1) = writeRam weA_ addrA_ datA1_ ram0
-      (wroteB,ram2) = writeRam weB_ addrB_ datB1_ ram1
-
-      outA1 = case conflict of
-        Just Conflict{cfRWA=IsDefined True} ->
-          deepErrorX "trueDualPortBlockRam: conflicting read/write queries"
-        Just Conflict{cfRWA=IsX} ->
-          deepErrorX "trueDualPortBlockRam: conflicting read/write queries"
-        _ -> fromMaybe (ram0 `Seq.index` addrA_) wroteA
-
-      outB1 = case conflict of
-        Just Conflict{cfRWB=IsDefined True} ->
-          deepErrorX "trueDualPortBlockRam: conflicting read/write queries"
-        Just Conflict{cfRWB=IsX} ->
-          deepErrorX "trueDualPortBlockRam: conflicting read/write queries"
-        _ -> fromMaybe (ram0 `Seq.index` addrB_) wroteB
-
-      outA2 = if enA_ then outA1 else prevA
-      outB2 = if enB_ then outB1 else prevB
-      (as2,bs2) = go ram2 (relativeTime - tB + tA) as1 bs1 outA2 outB2
-
-    -- 1 iteration here, as this is the slow clock.
-    goSlow _ prevB | enA_ = out0 `seqX` (out0 :- as2, bs2)
-     where
-      (wrote, !ram1) = writeRam weA_ addrA_ datA_ ram0
-      out0 = fromMaybe (ram1 `Seq.index` addrA_) wrote
-      (as2, bs2) = go ram1 (relativeTime + tA) as1 bs0 out0 prevB
-
-    goSlow prevA prevB = (prevA :- as2, bs2)
-      where
-        (as2,bs2) = go ram0 (relativeTime + tA) as1 bs0 prevA prevB
-
-    -- 1 or more iterations here, as this is the fast clock. First iteration
-    -- happens here.
-    goFast prevA _ | enB_ = out0 `seqX` (as2, out0 :- bs2)
-     where
-      (wrote, !ram1) = writeRam weB_ addrB_ datB_ ram0
-      out0 = fromMaybe (ram1 `Seq.index` addrB_) wrote
-      (as2, bs2) = go ram1 (relativeTime - tB) as0 bs1 prevA out0
-
-    goFast prevA prevB = (as2, prevB :- bs2)
-     where
-       (as2,bs2) = go ram0 (relativeTime - tB) as0 bs1 prevA prevB
+  updateRam :: Int -> MaybeX Bool -> a -> Seq a -> Seq a
+  updateRam addr writeEnable dat mem =
+    case writeEnable of
+      IsDefined False -> mem
+      IsDefined True -> Seq.update addr dat mem
+      IsX msg -> Seq.update addr dat $ deepErrorX $
+          "Write enable unknown; position" <> show addr <>
+        "\nWrite enable error message: " <> msg
