@@ -1,7 +1,7 @@
 {-|
 Copyright  :  (C) 2017, Google Inc.
                   2019, Myrtle Software Ltd
-                  2022-2023, QBayLogic B.V.
+                  2022-2024, QBayLogic B.V.
 License    :  BSD2 (see the file LICENSE)
 Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 
@@ -77,7 +77,7 @@ g :: Clock System
 g clk rst en s = writeToBiSignal s (mealy clk rst en counter (True, 0) (readFromBiSignal s))
 
 
--- | Connect the /f/ and /g/ circuits to the same bus
+-- | Connect the \/f\/ and \/g\/ circuits to the same bus
 topEntity
   :: Clock System
   -> Reset System
@@ -133,8 +133,8 @@ import           GHC.TypeLits               (KnownNat, Nat)
 import           GHC.Stack                  (HasCallStack)
 import           Data.Reflection            (Given (..))
 
--- | Used to specify the /default/ behavior of a \"BiSignal\", i.e. what value is
--- read when no value is being written to it.
+-- | Used to specify the /default/ behavior of a \"BiSignal\" in Haskell simulation, i.e.
+-- what value is read when no value is being written to it.
 data BiSignalDefault
   = PullUp
   -- ^ __inout__ port behaves as if connected to a pull-up resistor
@@ -203,13 +203,8 @@ type role BiSignalOut nominal nominal nominal
 --
 -- as it is not safe to coerce the default behaviour, synthesis domain or width
 -- of the data in the signal.
-#if MIN_VERSION_base(4,15,0) && !MIN_VERSION_base(4,17,0)
 data BiSignalOut (ds :: BiSignalDefault) (dom :: Domain) (n :: Nat)
   = BiSignalOut ![Signal dom (Maybe (BitVector n))]
-#else
-newtype BiSignalOut (ds :: BiSignalDefault) (dom :: Domain) (n :: Nat)
-  = BiSignalOut [Signal dom (Maybe (BitVector n))]
-#endif
 
 type instance HasDomain dom1 (BiSignalOut ds dom2 n) = DomEq dom1 dom2
 type instance TryDomain t (BiSignalOut ds dom n) = 'Found dom
@@ -277,8 +272,7 @@ writeToBiSignal#
   -> Signal d Bool
   -> Signal d (BitVector n)
   -> BiSignalOut ds d n
--- writeToBiSignal# = writeToBiSignal#
-writeToBiSignal# _ maybeSignal _ _ = BiSignalOut [maybeSignal]
+writeToBiSignal# bIn maybeSignal wEn val = BiSignalOut [bIn `seq` wEn `seq` val `seq` maybeSignal]
 -- See: https://github.com/clash-lang/clash-compiler/pull/2511
 {-# CLASH_OPAQUE writeToBiSignal# #-}
 {-# ANN writeToBiSignal# hasBlackBox #-}
