@@ -1,7 +1,7 @@
 {-|
 Copyright  :  (C) 2013-2016, University of Twente,
                   2016     , Myrtle Software Ltd,
-                  2021-2023, QBayLogic B.V.
+                  2021-2024, QBayLogic B.V.
 License    :  BSD2 (see the file LICENSE)
 Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 -}
@@ -184,23 +184,17 @@ type role Signed nominal
 --
 -- as it is not safe to coerce between different width Signed. To change the
 -- width, use the functions in the 'Clash.Class.Resize.Resize' class.
-#if MIN_VERSION_base(4,15,0) && !MIN_VERSION_base(4,17,0)
 data Signed (n :: Nat) =
     -- | The constructor, 'S', and the field, 'unsafeToInteger', are not
     -- synthesizable.
     S { unsafeToInteger :: !Integer}
-#else
-newtype Signed (n :: Nat) =
-    -- | The constructor, 'S', and the field, 'unsafeToInteger', are not
-    -- synthesizable.
-    S { unsafeToInteger :: Integer}
-#endif
   deriving (Data, Generic)
 
 {-# ANN S hasBlackBox #-}
 
 instance NFDataX (Signed n) where
   deepErrorX = errorX
+  ensureSpine = id
   rnfX = rwhnfX
 
 -- See: https://github.com/clash-lang/clash-compiler/pull/2511
@@ -881,7 +875,7 @@ instance (KnownNat n) => Ix (Signed n) where
   range (a, b) = [a..b]
   index ab@(a, b) x
     | inRange ab x = fromIntegral $ x - a
-    | otherwise = error $ printf "Index %d out of bounds (%d, %d) ab" x a b
+    | otherwise = error $ printf "Index (%d) out of range ((%d, %d))" x a b
   inRange (a, b) x = a <= x && x <= b
 
 -- | Shift left that ties to zero on negative shifts
