@@ -1,7 +1,7 @@
 {-|
 Copyright  :  (C) 2013-2016, University of Twente,
                   2016     , Myrtle Software Ltd,
-                  2021-2023, QBayLogic B.V.
+                  2021-2024, QBayLogic B.V.
 License    :  BSD2 (see the file LICENSE)
 Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 -}
@@ -200,17 +200,10 @@ type role Unsigned nominal
 --
 -- as it is not safe to coerce between different width Unsigned. To change the
 -- width, use the functions in the 'Clash.Class.Resize.Resize' class.
-#if MIN_VERSION_base(4,15,0) && !MIN_VERSION_base(4,17,0)
 data Unsigned (n :: Nat) =
     -- | The constructor, 'U', and the field, 'unsafeToNatural', are not
     -- synthesizable.
     U { unsafeToNatural :: !Natural }
-#else
-newtype Unsigned (n :: Nat) =
-    -- | The constructor, 'U', and the field, 'unsafeToNatural', are not
-    -- synthesizable.
-    U { unsafeToNatural :: Natural }
-#endif
   deriving (Data, Generic)
 
 {-# ANN U hasBlackBox #-}
@@ -240,6 +233,7 @@ instance ShowX (Unsigned n) where
 
 instance NFDataX (Unsigned n) where
   deepErrorX = errorX
+  ensureSpine = id
   rnfX = rwhnfX
 
 -- | None of the 'Read' class' methods are synthesizable.
@@ -794,7 +788,7 @@ instance (KnownNat n) => Ix (Unsigned n) where
   range (a, b) = [a..b]
   index ab@(a, b) x
     | inRange ab x = fromIntegral $ x - a
-    | otherwise = error $ printf "Index %d out of bounds (%d, %d) ab" x a b
+    | otherwise = error $ printf "Index (%d) out of range ((%d, %d))" x a b
   inRange (a, b) x = a <= x && x <= b
 
 unsignedToWord :: Unsigned WORD_SIZE_IN_BITS -> Word
