@@ -1,7 +1,8 @@
 {-|
 Copyright  :  (C) 2013-2016, University of Twente,
                   2017     , Myrtle Software Ltd
-                  2022-2023, QBayLogic B.V.
+                  2022-2025, QBayLogic B.V.
+                  2024,      Alex Mason
 License    :  BSD2 (see the file LICENSE)
 Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 -}
@@ -141,7 +142,7 @@ import Unsafe.Coerce              (unsafeCoerce)
 
 import Clash.Annotations.Primitive
   (Primitive(InlineYamlPrimitive), HDL(..), dontTranslate, hasBlackBox)
-import Clash.Magic (clashCompileError)
+import Clash.Magic (clashCompileError, clashSimulation)
 import Clash.Promoted.Nat
   (SNat (..), SNatLE (..), UNat (..), compareSNat, pow2SNat,
    snatProxy, snatToInteger, subSNat, withSNat, toUNat, natToInteger)
@@ -180,6 +181,8 @@ data Vec :: Nat -> Type -> Type where
   Nil  :: Vec 0 a
   Cons :: a -> Vec n a -> Vec (n + 1) a
 
+{-# COMPLETE Nil, (:>) #-}
+
 -- | In many cases, this Generic instance only allows generic
 -- functions/instances over vectors of at least size 1, due to the
 -- /n-1/ in the /Rep (Vec n a)/ definition.
@@ -408,7 +411,18 @@ singleton = (`Cons` Nil)
 >>> head (1:>2:>3:>Nil)
 1
 
-#if __GLASGOW_HASKELL__ >= 900
+#if __GLASGOW_HASKELL__ >= 910
+>>> head Nil
+<interactive>:...
+    • Couldn't match type ‘1’ with ‘0’
+      Expected: Vec (0 + 1) a
+        Actual: Vec 0 a
+    • In the first argument of ‘head’, namely ‘Nil’
+      In the expression: head Nil
+      In an equation for ‘it’: it = head Nil
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ >= 900
 >>> head Nil
 <BLANKLINE>
 <interactive>:...
@@ -443,7 +457,18 @@ head (x `Cons` _) = x
 >>> tail (1:>2:>3:>Nil)
 2 :> 3 :> Nil
 
-#if __GLASGOW_HASKELL__ >= 900
+#if __GLASGOW_HASKELL__ >= 910
+>>> tail Nil
+<interactive>:...
+    • Couldn't match type ‘1’ with ‘0’
+      Expected: Vec (0 + 1) a
+        Actual: Vec 0 a
+    • In the first argument of ‘tail’, namely ‘Nil’
+      In the expression: tail Nil
+      In an equation for ‘it’: it = tail Nil
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ >= 900
 >>> tail Nil
 <BLANKLINE>
 <interactive>:...
@@ -478,7 +503,18 @@ tail (_ `Cons` xs) = xs
 >>> last (1:>2:>3:>Nil)
 3
 
-#if __GLASGOW_HASKELL__ >= 900
+#if __GLASGOW_HASKELL__ >= 910
+>>> last Nil
+<interactive>:...
+    • Couldn't match type ‘1’ with ‘0’
+      Expected: Vec (0 + 1) a
+        Actual: Vec 0 a
+    • In the first argument of ‘last’, namely ‘Nil’
+      In the expression: last Nil
+      In an equation for ‘it’: it = last Nil
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ >= 900
 >>> last Nil
 <BLANKLINE>
 <interactive>:...
@@ -514,7 +550,18 @@ last (_ `Cons` y `Cons` ys) = last (y `Cons` ys)
 >>> init (1:>2:>3:>Nil)
 1 :> 2 :> Nil
 
-#if __GLASGOW_HASKELL__ >= 900
+#if __GLASGOW_HASKELL__ >= 910
+>>> init Nil
+<interactive>:...
+    • Couldn't match type ‘1’ with ‘0’
+      Expected: Vec (0 + 1) a
+        Actual: Vec 0 a
+    • In the first argument of ‘init’, namely ‘Nil’
+      In the expression: init Nil
+      In an equation for ‘it’: it = init Nil
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ >= 900
 >>> init Nil
 <BLANKLINE>
 <interactive>:...
@@ -762,8 +809,11 @@ merge x y = concat (transpose (x :> singleton y))
 -- >>> reverse (1:>2:>3:>4:>Nil)
 -- 4 :> 3 :> 2 :> 1 :> Nil
 reverse :: Vec n a -> Vec n a
-reverse Nil           = Nil
-reverse (x `Cons` xs) = reverse xs :< x
+reverse xs = go Nil xs
+ where
+  go :: i <= n => Vec (n - i) a -> Vec i a -> Vec n a
+  go a (y `Cons` ys) = go (y `Cons` a) ys
+  go a Nil = a
 -- See: https://github.com/clash-lang/clash-compiler/pull/2511
 {-# CLASH_OPAQUE reverse #-}
 {-# ANN reverse hasBlackBox #-}
@@ -809,15 +859,9 @@ imap f = go 0
 
 {- | Zip two vectors with a functions that also takes the elements' indices.
 
-#if __GLASGOW_HASKELL__ >= 900 && __GLASGOW_HASKELL__ < 904
 >>> izipWith (\i a b -> i + a + b) (2 :> 2 :> Nil)  (3 :> 3:> Nil)
 *** Exception: X: Clash.Sized.Index: result 2 is out of bounds: [0..1]
 ...
-#else
->>> izipWith (\i a b -> i + a + b) (2 :> 2 :> Nil)  (3 :> 3:> Nil)
-*** Exception: X: Clash.Sized.Index: result 3 is out of bounds: [0..1]
-...
-#endif
 >>> izipWith (\i a b -> extend (bitCoerce i) + a + b) (2 :> 2 :> Nil) (3 :> 3 :> Nil) :: Vec 2 (Unsigned 8)
 5 :> 6 :> Nil
 
@@ -1245,11 +1289,10 @@ postscanr f z xs = init (scanr f z xs)
 mapAccumL :: (acc -> x -> (acc,y)) -> acc -> Vec n x -> (acc,Vec n y)
 mapAccumL f acc xs = (acc',ys)
   where
-    accs  = acc `Cons` accs'
-    ws    = zipWith (flip f) xs (init accs)
-    accs' = map fst ws
-    ys    = map snd ws
-    acc'  = last accs
+    accs        = acc `Cons` accs'
+    ws          = zipWith (flip f) xs (init accs)
+    (accs', ys) = unzip ws
+    acc'        = last accs
 {-# INLINE mapAccumL #-}
 
 -- | The 'mapAccumR' function behaves like a combination of 'map' and 'foldr';
@@ -1266,11 +1309,10 @@ mapAccumL f acc xs = (acc',ys)
 mapAccumR :: (acc -> x -> (acc,y)) -> acc -> Vec n x -> (acc, Vec n y)
 mapAccumR f acc xs = (acc',ys)
   where
-    accs  = accs' :< acc
-    ws    = zipWith (flip f) xs (tail accs)
-    accs' = map fst ws
-    ys    = map snd ws
-    acc'  = head accs
+    accs        = accs' :< acc
+    ws          = zipWith (flip f) xs (tail accs)
+    (accs', ys) = unzip ws
+    acc'        = head accs
 {-# INLINE mapAccumR #-}
 
 -- | 'zip' takes two vectors and returns a vector of corresponding pairs.
@@ -1334,7 +1376,15 @@ zip7 = zipWith7 (,,,,,,)
 -- >>> unzip ((1,4):>(2,3):>(3,2):>(4,1):>Nil)
 -- (1 :> 2 :> 3 :> 4 :> Nil,4 :> 3 :> 2 :> 1 :> Nil)
 unzip :: Vec n (a,b) -> (Vec n a, Vec n b)
-unzip xs = (map fst xs, map snd xs)
+unzip xs
+  | clashSimulation = unzipSim xs
+  | otherwise = (map fst xs, map snd xs)
+ where
+  unzipSim :: Vec m (a,b) -> (Vec m a, Vec m b)
+  unzipSim Nil = (Nil, Nil)
+  unzipSim (~(a,b) `Cons` rest) =
+    let (as, bs) = unzipSim rest
+    in (a `Cons` as, b `Cons` bs)
 {-# INLINE unzip #-}
 
 -- | 'unzip3' transforms a vector of triplets into a vector of first components,
@@ -1343,31 +1393,55 @@ unzip xs = (map fst xs, map snd xs)
 -- >>> unzip3 ((1,4,5):>(2,3,6):>(3,2,7):>(4,1,8):>Nil)
 -- (1 :> 2 :> 3 :> 4 :> Nil,4 :> 3 :> 2 :> 1 :> Nil,5 :> 6 :> 7 :> 8 :> Nil)
 unzip3 :: Vec n (a,b,c) -> (Vec n a, Vec n b, Vec n c)
-unzip3 xs = ( map (\(x,_,_) -> x) xs
-            , map (\(_,y,_) -> y) xs
-            , map (\(_,_,z) -> z) xs
-            )
+unzip3 xs
+  | clashSimulation = unzip3Sim xs
+  | otherwise = ( map (\(x,_,_) -> x) xs
+                , map (\(_,y,_) -> y) xs
+                , map (\(_,_,z) -> z) xs
+                )
+ where
+  unzip3Sim :: Vec m (a,b,c) -> (Vec m a, Vec m b, Vec m c)
+  unzip3Sim Nil = (Nil, Nil, Nil)
+  unzip3Sim (~(a,b,c) `Cons` rest) =
+    let (as, bs, cs) = unzip3Sim rest
+    in (a `Cons` as, b `Cons` bs, c `Cons` cs)
 {-# INLINE unzip3 #-}
 
 -- | 'unzip4' takes a vector of quadruples and returns four vectors, analogous
 -- to 'unzip'.
 unzip4 :: Vec n (a,b,c,d) -> (Vec n a, Vec n b, Vec n c, Vec n d)
-unzip4 xs = ( map (\(w,_,_,_) -> w) xs
-            , map (\(_,x,_,_) -> x) xs
-            , map (\(_,_,y,_) -> y) xs
-            , map (\(_,_,_,z) -> z) xs
-            )
+unzip4 xs
+  | clashSimulation = unzip4Sim xs
+  | otherwise = ( map (\(w,_,_,_) -> w) xs
+                , map (\(_,x,_,_) -> x) xs
+                , map (\(_,_,y,_) -> y) xs
+                , map (\(_,_,_,z) -> z) xs
+                )
+ where
+  unzip4Sim :: Vec m (a,b,c,d) -> (Vec m a, Vec m b, Vec m c, Vec m d)
+  unzip4Sim Nil = (Nil, Nil, Nil, Nil)
+  unzip4Sim (~(a,b,c,d) `Cons` rest) =
+    let (as, bs, cs, ds) = unzip4Sim rest
+    in (a `Cons` as, b `Cons` bs, c `Cons` cs, d `Cons` ds)
 {-# INLINE unzip4 #-}
 
 -- | 'unzip5' takes a vector of five-tuples and returns five vectors, analogous
 -- to 'unzip'.
 unzip5 :: Vec n (a,b,c,d,e) -> (Vec n a, Vec n b, Vec n c, Vec n d, Vec n e)
-unzip5 xs = ( map (\(v,_,_,_,_) -> v) xs
-            , map (\(_,w,_,_,_) -> w) xs
-            , map (\(_,_,x,_,_) -> x) xs
-            , map (\(_,_,_,y,_) -> y) xs
-            , map (\(_,_,_,_,z) -> z) xs
-            )
+unzip5 xs
+  | clashSimulation = unzip5Sim xs
+  | otherwise = ( map (\(v,_,_,_,_) -> v) xs
+                , map (\(_,w,_,_,_) -> w) xs
+                , map (\(_,_,x,_,_) -> x) xs
+                , map (\(_,_,_,y,_) -> y) xs
+                , map (\(_,_,_,_,z) -> z) xs
+                )
+ where
+  unzip5Sim :: Vec m (a,b,c,d,e) -> (Vec m a, Vec m b, Vec m c, Vec m d, Vec m e)
+  unzip5Sim Nil = (Nil, Nil, Nil, Nil, Nil)
+  unzip5Sim (~(a,b,c,d,e) `Cons` rest) =
+    let (as, bs, cs, ds, es) = unzip5Sim rest
+    in (a `Cons` as, b `Cons` bs, c `Cons` cs, d `Cons` ds, e `Cons` es)
 {-# INLINE unzip5 #-}
 
 -- | 'unzip6' takes a vector of six-tuples and returns six vectors, analogous
@@ -1375,13 +1449,21 @@ unzip5 xs = ( map (\(v,_,_,_,_) -> v) xs
 unzip6
   :: Vec n (a,b,c,d,e,f)
   -> (Vec n a, Vec n b, Vec n c, Vec n d, Vec n e, Vec n f)
-unzip6 xs = ( map (\(u,_,_,_,_,_) -> u) xs
-            , map (\(_,v,_,_,_,_) -> v) xs
-            , map (\(_,_,w,_,_,_) -> w) xs
-            , map (\(_,_,_,x,_,_) -> x) xs
-            , map (\(_,_,_,_,y,_) -> y) xs
-            , map (\(_,_,_,_,_,z) -> z) xs
-            )
+unzip6 xs
+  | clashSimulation = unzip6Sim xs
+  | otherwise = ( map (\(u,_,_,_,_,_) -> u) xs
+                , map (\(_,v,_,_,_,_) -> v) xs
+                , map (\(_,_,w,_,_,_) -> w) xs
+                , map (\(_,_,_,x,_,_) -> x) xs
+                , map (\(_,_,_,_,y,_) -> y) xs
+                , map (\(_,_,_,_,_,z) -> z) xs
+                )
+ where
+  unzip6Sim :: Vec m (a,b,c,d,e,f) -> (Vec m a, Vec m b, Vec m c, Vec m d, Vec m e, Vec m f)
+  unzip6Sim Nil = (Nil, Nil, Nil, Nil, Nil, Nil)
+  unzip6Sim (~(a,b,c,d,e,f) `Cons` rest) =
+    let (as, bs, cs, ds, es, fs) = unzip6Sim rest
+    in (a `Cons` as, b `Cons` bs, c `Cons` cs, d `Cons` ds, e `Cons` es, f `Cons` fs)
 {-# INLINE unzip6 #-}
 
 -- | 'unzip7' takes a vector of seven-tuples and returns seven vectors, analogous
@@ -1389,14 +1471,22 @@ unzip6 xs = ( map (\(u,_,_,_,_,_) -> u) xs
 unzip7
   :: Vec n (a,b,c,d,e,f,g)
   -> (Vec n a, Vec n b, Vec n c, Vec n d, Vec n e, Vec n f, Vec n g)
-unzip7 xs = ( map (\(t,_,_,_,_,_,_) -> t) xs
-            , map (\(_,u,_,_,_,_,_) -> u) xs
-            , map (\(_,_,v,_,_,_,_) -> v) xs
-            , map (\(_,_,_,w,_,_,_) -> w) xs
-            , map (\(_,_,_,_,x,_,_) -> x) xs
-            , map (\(_,_,_,_,_,y,_) -> y) xs
-            , map (\(_,_,_,_,_,_,z) -> z) xs
-            )
+unzip7 xs
+  | clashSimulation = unzip7Sim xs
+  | otherwise = ( map (\(t,_,_,_,_,_,_) -> t) xs
+                , map (\(_,u,_,_,_,_,_) -> u) xs
+                , map (\(_,_,v,_,_,_,_) -> v) xs
+                , map (\(_,_,_,w,_,_,_) -> w) xs
+                , map (\(_,_,_,_,x,_,_) -> x) xs
+                , map (\(_,_,_,_,_,y,_) -> y) xs
+                , map (\(_,_,_,_,_,_,z) -> z) xs
+                )
+ where
+  unzip7Sim :: Vec m (a,b,c,d,e,f,g) -> (Vec m a, Vec m b, Vec m c, Vec m d, Vec m e, Vec m f, Vec m g)
+  unzip7Sim Nil = (Nil, Nil, Nil, Nil, Nil, Nil, Nil)
+  unzip7Sim (~(a,b,c,d,e,f,g) `Cons` rest) =
+    let (as, bs, cs, ds, es, fs, gs) = unzip7Sim rest
+    in (a `Cons` as, b `Cons` bs, c `Cons` cs, d `Cons` ds, e `Cons` es, f `Cons` fs, g `Cons` gs)
 {-# INLINE unzip7 #-}
 
 
@@ -1490,7 +1580,19 @@ replace i y xs = replace_int xs (fromEnum i) y
 >>> take d0               (1:>2:>Nil)
 Nil
 
-#if __GLASGOW_HASKELL__ == 906
+#if __GLASGOW_HASKELL__ >= 910
+>>> take d4               (1:>2:>Nil)
+<interactive>:...
+    • Couldn't match type ‘4 + n0’ with ‘2’
+      Expected: Vec (4 + n0) a
+        Actual: Vec (1 + 1) a
+      The type variable ‘n0’ is ambiguous
+    • In the second argument of ‘take’, namely ‘(1 :> 2 :> Nil)’
+      In the expression: take d4 (1 :> 2 :> Nil)
+      In an equation for ‘it’: it = take d4 (1 :> 2 :> Nil)
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ == 906
 >>> take d4               (1:>2:>Nil)
 <BLANKLINE>
 <interactive>:...
@@ -1549,7 +1651,16 @@ takeI = withSNat take
 >>> drop d0               (1:>2:>Nil)
 1 :> 2 :> Nil
 
-#if __GLASGOW_HASKELL__ == 906
+#if __GLASGOW_HASKELL__ >= 910
+>>> drop d4               (1:>2:>Nil)
+<interactive>:...: error:...
+    • Couldn't match...type ‘4 + n0...
+      The type variable ‘n0’ is ambiguous
+    • In the first argument of ‘print’, namely ‘it’
+      In a stmt of an interactive GHCi command: print it
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ == 906
 >>> drop d4               (1:>2:>Nil)
 <BLANKLINE>
 <interactive>:...: error:...
@@ -1683,10 +1794,11 @@ iterate SNat = iterateI
 --
 -- <<doc/iterate.svg>>
 iterateI :: forall n a. KnownNat n => (a -> a) -> a -> Vec n a
-iterateI f a = xs
-  where
-    xs = init (a `Cons` ws)
-    ws = map f (lazyV xs)
+iterateI f = iterateU (toUNat (SNat @n))
+ where
+  iterateU :: forall m. UNat m -> a -> Vec m a
+  iterateU UZero _ = Nil
+  iterateU (USucc s) a = a `Cons` iterateU s (f a)
 -- See: https://github.com/clash-lang/clash-compiler/pull/2511
 {-# CLASH_OPAQUE iterateI #-}
 {-# ANN iterateI (InlineYamlPrimitive [VHDL,Verilog,SystemVerilog] [I.__i|
@@ -1943,9 +2055,7 @@ gather xs = map (xs!!)
 
 -- | \"'interleave' @d xs@\" creates a vector:
 --
--- @
--- \<x_0,x_d,x_(2d),...,x_1,x_(d+1),x_(2d+1),...,x_(d-1),x_(2d-1),x_(3d-1)\>
--- @
+-- > <x_0,x_d,x_(2d),...,x_1,x_(d+1),x_(2d+1),...,x_(d-1),x_(2d-1),x_(3d-1)>
 --
 -- >>> let xs = 1 :> 2 :> 3 :> 4 :> 5 :> 6 :> 7 :> 8 :> 9 :> Nil
 -- >>> interleave d3 xs
@@ -2366,7 +2476,27 @@ defined in the instance 'Clash.Class.Num.ExtendingNum' instance of 'Index'.
 However, we cannot simply use 'fold' to create a tree-structure of
 'Clash.Class.Num.add'es:
 
-#if __GLASGOW_HASKELL__ >= 900
+#if __GLASGOW_HASKELL__ >= 910
+>>> :{
+let populationCount' :: (KnownNat (n+1), KnownNat (n+2))
+                     => BitVector (n+1) -> Index (n+2)
+    populationCount' = fold add . map fromIntegral . bv2v
+:}
+<interactive>:...
+    • Couldn't match type: ((n + 2) + (n + 2)) - 1
+                     with: n + 2
+      Expected: Index (n + 2) -> Index (n + 2) -> Index (n + 2)
+        Actual: Index (n + 2)
+                -> Index (n + 2) -> AResult (Index (n + 2)) (Index (n + 2))
+    • In the first argument of ‘fold’, namely ‘add’
+      In the first argument of ‘(.)’, namely ‘fold add’
+      In the expression: fold add . map fromIntegral . bv2v
+    • Relevant bindings include
+        populationCount' :: BitVector (n + 1) -> Index (n + 2)
+          (bound at ...)
+<BLANKLINE>
+
+#elif __GLASGOW_HASKELL__ >= 900
 >>> :{
 let populationCount' :: (KnownNat (n+1), KnownNat (n+2))
                      => BitVector (n+1) -> Index (n+2)
