@@ -2,17 +2,17 @@
 Copyright  :  (C) 2016,      University of Twente,
                   2017,      QBayLogic, Google Inc.
                   2017-2019, Myrtle Software Ltd,
-                  2021-2023, QBayLogic B.V.
+                  2021-2025, QBayLogic B.V.
 License    :  BSD2 (see the file LICENSE)
 Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 
 'XException': An exception for uninitialized values
 
->>> show (errorX "undefined" :: Integer, 4 :: Int)
-"(*** Exception: X: undefined
+>>> show (errorX "No value here" :: Integer, 4 :: Int)
+"(*** Exception: X: No value here
 CallStack (from HasCallStack):
 ...
->>> showX (errorX "undefined" :: Integer, 4 :: Int)
+>>> showX (errorX "No value here" :: Integer, 4 :: Int)
 "(undefined,4)"
 -}
 
@@ -63,6 +63,7 @@ import qualified Data.List.Infinite  as Inf
 import           Data.List.Infinite  (Infinite (..))
 import           Data.List.NonEmpty  (NonEmpty)
 import           Data.Ord            (Down (Down))
+import           Data.Proxy          (Proxy)
 import           Data.Ratio          (Ratio, numerator, denominator)
 import qualified Data.Semigroup      as SG
 import qualified Data.Monoid         as M
@@ -357,11 +358,11 @@ isX a =
 -- | Like the 'Show' class, but values that normally throw an 'XException' are
 -- converted to @undefined@, instead of error'ing out with an exception.
 --
--- >>> show (errorX "undefined" :: Integer, 4 :: Int)
--- "(*** Exception: X: undefined
+-- >>> show (errorX "No value here" :: Integer, 4 :: Int)
+-- "(*** Exception: X: No value here
 -- CallStack (from HasCallStack):
 -- ...
--- >>> showX (errorX "undefined" :: Integer, 4 :: Int)
+-- >>> showX (errorX "No value here" :: Integer, 4 :: Int)
 -- "(undefined,4)"
 --
 -- Can be derived using 'GHC.Generics':
@@ -397,6 +398,8 @@ printX :: ShowX a => a -> IO ()
 printX x = putStrLn $ showX x
 
 instance ShowX ()
+-- | @since 1.8.2
+instance ShowX (Proxy a)
 instance ShowX a => ShowX (Identity a)
 instance ShowX a => ShowX (Const a b)
 instance (ShowX (f a), ShowX (g a)) => ShowX (Product f g a)
@@ -585,6 +588,8 @@ instance NFDataX a => NFDataX [a]
 instance NFDataX a => NFDataX (NonEmpty a)
 instance (NFDataX a, NFDataX b) => NFDataX (Either a b)
 instance NFDataX a => NFDataX (Maybe a)
+-- | @since 1.8.2
+instance NFDataX (Proxy a)
 instance NFDataX a => NFDataX (Identity a)
 instance NFDataX a => NFDataX (Const a b)
 instance (NFDataX (f a), NFDataX (g a)) => NFDataX (Product f g a)
