@@ -202,19 +202,21 @@ printR
   :: Foldable f
   => f (Bool, (Int, Bool))
   -> IO ()
-printR r = do
-  let (first:rest) = P.map show $ sampleR 60 r
-  putStrLn $ (P.foldl (\b a -> b <> "  , " <> a <> "\n")
-                ("  [ " <> first <> "\n") rest) <> "  ]"
+printR r = case P.map show $ sampleR 60 r of
+  (first:rest) ->
+    putStrLn $ (P.foldl (\b a -> b <> "  , " <> a <> "\n")
+                  ("  [ " <> first <> "\n") rest) <> "  ]"
+  _ -> error "impossible"
 
 printW
   :: (Foldable f, NFDataX a, Show a)
   => f a
   -> IO ()
-printW w = do
-  let (first:rest) = P.map show $ sampleN 60 w
-  putStrLn $ (P.foldl (\b a -> b <> "  , " <> a <> "\n")
-                ("  [ " <> first <> "\n") rest) <> "  ]"
+printW w = case P.map show $ sampleN 60 w of
+  (first:rest) ->
+    putStrLn $ (P.foldl (\b a -> b <> "  , " <> a <> "\n")
+                  ("  [ " <> first <> "\n") rest) <> "  ]"
+  _ -> error "impossible"
 
 testR
   :: Foldable f
@@ -277,7 +279,7 @@ test2R1 = testR sync2R1
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (False,(Just 1,False))
   , (False,(Just 1,False))
@@ -287,7 +289,7 @@ test2R1 = testR sync2R1
   , (True,(Just 3,False))
   , (True,(Just 4,False))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 5,True))
   , (False,(Just 5,True))
   , (False,(Just 5,False))
   , (False,(Just 5,False))
@@ -362,7 +364,7 @@ test2R2 = testR sync2R2
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (False,(Just 1,False))
   , (False,(Just 1,False))
@@ -372,7 +374,7 @@ test2R2 = testR sync2R2
   , (True,(Just 2,False))
   , (True,(Just 3,False))
   , (True,(Just 4,False))
-  , (False,(Nothing,True))
+  , (False,(Just 5,True))
   , (False,(Just 5,True))
   , (False,(Just 5,False))
   , (False,(Just 5,False))
@@ -452,7 +454,7 @@ test2R3 = testR sync2R3
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (False,(Just 1,False))
   , (False,(Just 1,False))
@@ -462,7 +464,7 @@ test2R3 = testR sync2R3
   , (True,(Just 3,False))
   , (True,(Just 4,False))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 5,True))
   , (False,(Just 5,True))
   , (False,(Just 5,False))
   , (False,(Just 5,False))
@@ -540,7 +542,7 @@ test2R4 = testR sync2R4
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (False,(Just 1,False))
   , (False,(Just 1,False))
@@ -550,7 +552,7 @@ test2R4 = testR sync2R4
   , (True,(Just 2,False))
   , (True,(Just 3,False))
   , (True,(Just 4,False))
-  , (False,(Nothing,True))
+  , (False,(Just 5,True))
   , (False,(Just 5,True))
   , (False,(Just 5,False))
   , (False,(Just 5,False))
@@ -641,7 +643,7 @@ test4R5 = testR sync4R5
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (False,(Just 1,False))
   , (False,(Just 1,False))
@@ -776,7 +778,7 @@ test4R6 = testR sync4R6
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (False,(Just 1,False))
   , (False,(Just 1,False))
@@ -784,7 +786,7 @@ test4R6 = testR sync4R6
   , (True,(Just 2,False))
   , (True,(Just 3,False))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 4,True))
   , (False,(Just 4,True))
   , (False,(Just 4,False))
   , (False,(Just 4,False))
@@ -909,7 +911,7 @@ test6R7 = testR sync6R7
   , (False,(Nothing,True))
   , (False,(Nothing,True))
   , (False,(Nothing,True))
-  , (False,(Nothing,True))
+  , (False,(Just 1,True))
   , (False,(Just 1,True))
   , (True,(Just 1,False))
   , (False,(Just 2,True))
@@ -1051,8 +1053,8 @@ fifoOperations racts wacts = (bundle (rAllDone, rel), bundle (wAllDone, wel))
   wclk = clockGen @wdom
   -- Not resetting makes the test easier to interpret and actual proper testing
   -- of reset behaviour is a lot more involved.
-  noRRst = unsafeFromHighPolarity @rdom (pure False)
-  noWRst = unsafeFromHighPolarity @wdom (pure False)
+  noRRst = unsafeFromActiveHigh @rdom (pure False)
+  noWRst = unsafeFromActiveHigh @wdom (pure False)
   (wdone, wact) =
     unbundle $ fromList $ P.zip (P.repeat False) wacts <> P.repeat (True, WNoOp)
   (rdone, ract) =
